;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x9b06bebbdee2ecb44864b0c513dfe1b2 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x2ff7bdfe51517c14ee5fb1da27641ad1 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x2e22a1a080f5413532646aa4e26905b6 . "./code.cg2js/cg-grid-widget.fasl")
(#xc3a4bfbd4bb1536bf0cbaac69ace1b04 . "./code.cg2js/cg-utility-dialog.fasl")
(#x3a5f0f896aa29f61381011a32f1ec5f4 . "./code.cg2js/cg-wrap-string.fasl")
(#x0ca180f0471fa87ea4792f89b416fcdc . "./code.cg2js/cg-editable-text.fasl")
(#xf7c925c930ddfdcf667b89cd8db53118 . "./code.cg2js/cg-base.fasl")
(#xc90edddb896944ba54c5082bfde299ef . "./code.cg2js/cg-lisp-text.fasl")
(#x79c3ff14b0086cfd53905eec6b8fca21 . "./code.cg2js/cg-intersect.fasl")
(#x441d8c49ab81b8f7535c4212d20d8033 . "./code.cg2js/cg-choose-printer.fasl")
(#x291d5223ce4889fcc329eba0a8dc34bd . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x03f3952bc576ea07faa74b25cf025c12 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0a8484ff1a824dba9a57d4fcd4aef6dc . "./code.cg2js/cg-font-dialog.fasl")
(#xa35562f375b89097f7aff3d9249272e6 . "./code.cg2js/cg-comtab.fasl")
(#xa986cef3c7d1651f6c93c8d666f13d72 . "./code.cg2js/cg-tooltip.fasl")
(#x80c9339fd026b1fc1cda290d43bfcba3 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xfc1b561a9675a1ccbf8087ab421f9235 . "./code.cg2js/cg-get-pixmap.fasl")
(#xaff70342d801199824b0889de0c80206 . "./code.cg2js/cg-status-bar.fasl")
(#x688aac2b1da68e846b00576d09336ab3 . "./code.cg2js/cg-tab-control.fasl")
(#x26b01ce1fed7013d627a24e71c68ed48 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7a041b8daf6996126fe5b9a92b83728b . "./code.cg2js/cg-list-view.fasl")
(#x193713b82cda582e4e21bcfb1b72ca02 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xebbc6d01b8c5ccb0f10b5867f2d074eb . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x2ac4ff2c8f307cedea8b287ab3ba3103 . "./code.cg2js/cg-project-window.fasl")
(#xb930f4c28cc576f571b5f99cd452a75f . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x5b0771a6db90f80423f49080a180fb65 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x2f9f4a431e4573f556da04a3543b5327 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc5b51871fc6157b39b1d61bc044c12a7 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x79663c8f0e16317f3da8d70743e2b3db . "./code.cg2js/cg-menu.fasl")
(#x52e2508e28e002a2192b766996ec7d0d . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xcf0750c71fffa151db6e27e38fead5e8 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xd0db7b09ddf5a73c50690f000a4cd9fb . "./code.cg2js/cg-split-bar.fasl")
(#xb8a261f0989c83d2d2ac87cf4417c2a4 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x3d908b45d340c2327fc7a837cc0d5ee7 . "./code.cg2js/cg-timer.fasl")
(#x0028141bd9bba5be73bfd8c294086736 . "./code.cg2js/cg-text-or-combo.fasl")
(#x753a8e3cc5eaf4f3dfb0e9ad5b41c449 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xe11aab371191bd602cc0cb0e367f41d4 . "./code.cg2js/cg-string-dialog.fasl")
(#x849d5a03a906d37b5c510ff45512a240 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x5d83de22377911eca098fd6e6396ad12 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x3920e146cf991297218162891073ed3f . "./code.cg2js/cg-outline.fasl")
(#x8bb7376fc1cf7c16823428d8bc4d6821 . "./code.cg2js/cg-lettered-menu.fasl")
(#xbbe4123f4b2808e5db1795872762857f . "./code.cg2js/cg-graphics-context.fasl")
(#x3e678333e204d36a639fc24ef7e565a1 . "./code.cg2js/cg-item-list.fasl")
(#x7b2c52d0c9ac61757464ae275a3dc604 . "./code.cg2js/cg-caret.fasl")
(#xed70e60f95fbe1bb35a43a8ee69491ce . "./code.cg2js/cg-printing.fasl")
(#x89d4c51a489e9b4125aeade23b54f8f2 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x573f42df8d1a7ee07cd1d8a95f7b7c1b . "./code.cg2js/cg-progress-indicator.fasl")
(#xf737130d3cf77d90919e63f836fa7ca0 . "./code.cg2js/cg-dde-utils.fasl")
(#x306a8a411fb4755c917a051ab4fefd72 . "./code.cg2js/cg-plot-widget.fasl")
(#xb8a1aef12e84003d123b22c066f70ac1 . "./code.cg2js/cg-os-window.fasl")
(#xded2f16520cf2ba0b50279b1947cb652 . "./code.cg2js/cg-calendar.fasl")
(#x41005c2c122e1017f91968f9062ee3f5 . "./code.cg2js/cg-chart-widget.fasl")
(#xe3cd9d931784f3400f56ed56dead2ad4 . "./code.cg2js/cg-file-dialog.fasl")
(#x65a49b653aa931cba70bb234934f2e07 . "./code.cg2js/cg-nodes.fasl")
(#x79654d75c5e383b6ff406d35922cf35a . "./code.cg2js/cg-lamp.fasl")
(#x9a27914d3f54176cae0d1fe7befc079a . "./code.cg2js/cg-html-dialog.fasl")
(#x934d9240c5d896d904a4b5bbff96105a . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x9d69f386b33b803ce3cc48b124e75987 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x27e09fddde59277ddf10057b29f1872f . "./code.cg2js/cg-choice-list.fasl")
(#x92a3b2baa6030a141298354753502ebd . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8158f05efac189799cc03e21e458c9e0 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xb20b2a38daf3bc5a374c33ba04376840 . "./code.cg2js/cg-hotspot.fasl")
(#x44306be78755656ab574b6f51a0a65b6 . "./code.cg2js/cg-property.fasl")
(#x69e54ca714bff948d385ac84ee9cb071 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb4aeb475ba5ea4475c12f8231f290e99 . "./code.cg2js/cg-template-string.fasl")
(#x5bf7a03ca1bb4f598736a0e6846f4848 . "./code.cg2js/cg-palette.fasl")
(#x99ff3b441e3b147bf2fc966c4cdf7674 . "./code.cg2js/cg-selected-object.fasl")
(#x73f6c0d89ff0777d68a73efc65880893 . "./code.cg2js/cg-up-down-control.fasl")
(#xacb4c298e9046340de703940076637fd . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x47605aa8a430ee63a936dbde9b19063c . "./code.cg2js/cg-layout.fasl")
(#xa2aa78ef2c82ea9651f529b25023a4ff . "./code.cg2js/cg-get-position.fasl")
(#xeb88fe2b6683ea2421c732e38c199cb7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x2a3352a9875eb80b9f56bae5558cc1b7 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xb4217f9e4062e97e842d478785096d0a . "./code.cg2js/cg-object-editor.fasl")
(#xc5992731d48733182908e7835d6d67e1 . "./code.cg2js/cg-class-grid.fasl")
(#x69789b585f6701efcf485ef7d52460bb . "./code.cg2js/cg-yes-no-string.fasl")
(#x096497c77af07db3addd4b0b8c9fdaec . "./code.cg2js/cg-toggling-widget.fasl")
(#xb9d0159d476703a65de56e6c9329a519 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe83e3e2e5bd281b238422074bc7a600a . "./code.cg2js/cg-button.fasl")
(#x545e8604a1452cbc017eb7c74cbdefd7 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x147688a3642f6e6a7f05421a7988504b . "./code.cg2js/cg-scroll-bar.fasl")
(#xee37ef6dedd6317cb28661a216c44d69 . "./code.cg2js/cg-combo-box.fasl")
(#xe7756668670936d6f01fac427744e3fc . "./code.cg2js/cg-web-browser.fasl")
(#x63b95bcecbf9dabc384c4d684a9e85bc . "./code.cg2js/cg-drawable.fasl")
(#xbe6cb29e7c78067b7651302798421a27 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xf0ab8d2679ee93996fd49619710ffe6f . "./code.cg2js/cg-message-dialog.fasl")
(#xe41279c44f8c27a7899052ff02363692 . "./code.cg2js/cg-text-widget.fasl")
(#x00dba47d1b938b1a47820bfc87c00c4b . "./code.cg2js/cg-static-text.fasl")
(#x41078f92f71a4e30dfa5560dde18b7cf . "./code.cg2js/cg-class-support.fasl")
(#x6358b37be6e1f49c694c7fa362546534 . "./code.cg2js/cg.fasl")
(#x937eece37454a6159a7e2c208249541c . "./code.cg2js/cg-directory-dialog.fasl")
(#xf2705c9b84185d9558199178886b6de5 . "./code.cg2js/cg-clipboard.fasl")
(#x52a02aae46fe8dd927265dc2633f6c2a . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xadbf64ec2effe7d0613d86edcfe228c0 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x3970d1c994e99190619f700c3bc2e486 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xb2245cd7d0fd9de1933785336d90015e . "./code.cg2js/cg-check-box.fasl")
(#xb247a47e50a3ac3bf4f4fc63eedad92c . "./code.cg2js/cg-color-dialog.fasl")
(#x51b142f50558cec5ca894f817f5c9f40 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xdfbac63d2e1af78a069177f465956f4e . "./code.cg2js/cg-dialog-item.fasl")
(#x6e9ec72883766a1ce47f75662a4ff929 . "./code.cg2js/cg-edit-in-place.fasl")
(#xb06516fb67eacf2e12f5c92d896e10ef . "./code.cg2js/cg-tray.fasl")
(#x70a7bea09271a1ee80f76976e112ff9c . "./code.cg2js/cg-paren-matching.fasl")
(#xacee1a61df55d7bb93149d89a94bc83f . "./code.cg2js/cg-player.fasl")
(#x848da4299e0a8552e429928808b16afd . "./code.cg2js/cg-yes-no-list.fasl")
(#x1df695a3d52fa31074cc0b18e1785f94 . "./code.cg2js/cg-common-control.fasl")
(#x6d393ac79ae8ca3291c470ae488a072e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x714781bc6df7668de18845415e5199a2 . "./code.cg2js/cg-os-widget.fasl")
(#x74c69e6ef1555994048df64499e22d07 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbc0a655eef3677027b9f284aefa1ee03 . "./code.cg2js/cg-trackbar.fasl")
(#xe129ccb4e83c618e8d878e8cef609fc9 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xa95e1011fa1940e506748ea89d7b9eaf . "./code.cg2js/cg-html-widget.fasl")
(#x24258e30408e539afbc023fdfe663727 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x39d1bd1a86b37d1ab468ceb6409ba8f7 . "./code.cg2js/cg-acache.fasl")
(#xa7c0db297da84800290e291323ec4845 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xf4edbc652d39afb6683f51ff010a0e6a . "./code.cg2js/cg-pixmap-widget.fasl")
(#xb3d4fb8ea1422bd77854fe18d2a42451 . "./code.cg2js/cg-icon.fasl")
(#x0d4920086b263e94014bb757c25b1e9a . "./code.cg2js/cg-pixmap.fasl")
(#xc458158312b9bc74fee318c8c4fda142 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x7f3dcb994285ff44cd54676383a96875 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xd6d2f2b0e8f09c9fdb358d91d447d0dc . "./code.cg2js/cg-toolbar.fasl")
(#xb31f38b17535fe5f8e40ab2096da44b5 . "./code.cg2js/cg-radio-button.fasl")
(#xfb7ebd4e019cd702878ecfa2ea3a79b0 . "./code.cg2js/cg-curve.fasl")
(#x2f2294d6b946297fcc55ad41c330a22c . "./code.cg2js/cg-picture-widget.fasl")
(#x99ce8802812ecade4252f2a88f5fbe00 . "./code.cg2js/cg-dev.fasl")
(#xd373e88f7e34e436818a8cf1f74d4581 . "./code.cg2js/cg-group-box.fasl")
(#xf073f978c2a1a475bef743571b0c0c41 . "./code.cg2js/cg-fill-texture.fasl")
(#x134cfed951483cb4121f407c7e45f1f5 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x2d8b5eeb6e1d012260ca9e420d0c6504 . "./code.idejs/ide-keyshort.fasl")
(#x3ce245090dc1fe5beedff38ba40f173c . "./code.idejs/ide-form.fasl")
(#x4e2fd18594f0df0e8d2391d0e326e1a3 . "./code.idejs/ide-find-in-files.fasl")
(#xff380e6aff45a62cd1b368fef944a523 . "./code.idejs/ide-class-grapher.fasl")
(#x382adbecdc1a63277e3a166a81fdb899 . "./code.idejs/ide-base.fasl")
(#xa2edd97b046ccb746cc342e47bebcd7e . "./code.idejs/ide-class-browser.fasl")
(#x5937d4f79ecfee778440f46940054afa . "./code.idejs/ide-find-definitions.fasl")
(#x6e3af72e774a875c91acafb2a5de462b . "./code.idejs/ide-package-list.fasl")
(#x44b3029d3be199a82555abfce84895b2 . "./code.idejs/ide-grapher.fasl")
(#xc2551b55a0dfa66c85679ab1c84513a2 . "./code.idejs/ide-dev.fasl")
(#xe7dfabc44f46a5a378386d149c56ce0b . "./code.idejs/ide.fasl")
(#xb4e38ed60868d9462b24ba2eb522eb0d . "./code.idejs/ide-trace.fasl")
(#x70ab1315f4fa187a525fc187a3110237 . "./code.idejs/ide-apropos.fasl")
(#x100964ad43430138488c7aaecc0a273e . "./code.idejs/ide-inspector.fasl")
(#xa22b689aabc951cf5ba0753299bb4336 . "./code.idejs/ide-acache.fasl")
(#x9cb9fe91175e47c562b0ff7f4859b15a . "./code.idejs/ide-project.fasl")
(#x8fc162b310d5d33314b16439f4836dc7 . "./code.idejs/ide-window-list.fasl")
(#x1ba44921180ada3493d3f62e0ce906ec . "./code.idejs/ide-intro.fasl")
(#x3865874f2118179da05bf4afdcf5614c . "./code.idejs/ide-clipboard.fasl")
(#x0fd99a1024fec8146c4b110db95906f3 . "./code.idejs/ide-editor.fasl")
(#x240c071d2e3ce5cb7d0bdff13f88b95e . "./code.idejs/ide-process-browser.fasl")
(#xe04b9b8cbd4cc4605932de44ddbd25b4 . "./code.idejs/ide-patch-dialog.fasl")
(#x4ccc6a9a6bb7572c197fbb4d5f502b43 . "./code.idejs/ide-navigator.fasl")
(#x7e8a653f63c59e56c07642baa40b4a7f . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
